<?php

namespace Drupal\menu_to_taxonomy\Plugin\migrate\source\menu_to_taxonomy;

use Drupal\migrate\Row;
use Drupal\migrate_drupal\Plugin\migrate\source\DrupalSqlBase;

/**
 * Drupal 7 menu to taxonomy entity configurations from database.
 *
 * @MigrateSource(
 *   id = "d7_menu_to_taxonomy_settings",
 *   source_module = "menu_to_taxonomy"
 * )
 */
class MenuToTaxonomySettings extends DrupalSqlBase {

  /**
   * {@inheritdoc}
   */
  public function query() {
    // Since this is a menu to taxonomy config entity, we need id, label,
    // menu id and vocabulary id. As such, we need to get a query with this
    // info that will be processed when creating the rows. Since D7 uses variables
    // as configurations, we will process the rows generated by this query on a iterator.
    $query = $this->select('variable', 'v');
    $query->fields('v', ['name', 'value']);
    $query->condition('v.name', 'menu_to_taxonomy_vocab_menu_%', 'LIKE');
    $query->innerJoin('taxonomy_vocabulary', 'tv', "tv.machine_name=SUBSTRING_INDEX(v.name, 'menu_to_taxonomy_vocab_menu_', -1)");
    $query->fields('tv', ['vid']);
    $query->addField('tv', 'machine_name', 'vocabulary_machine_name');
    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function fields() {
    return [
      'id' => $this->t('The config entity ID.'),
      'label' => $this->t('The config entity label.'),
      'vid' => $this->t('The vocabulary machine name.'),
      'mid' => $this->t('The menu machine name.'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getIds() {
    return [
      'name' => [
        'type' => 'string',
        'alias' => 'v'
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function prepareRow(Row $row) {
    // Gets the menu id.
    if ($mid = unserialize($row->get('value'))) {
      // We remove the "-menu" suffix if the menu
      // is the main menu.
      $id = $mid === 'main-menu' ? 'main' : $mid;
      // Replace dashes with underscores for the config
      // entity id.
      $replaced_id = str_replace('-', '_', $id);
      // We grab the vocabulary id from the source "name" property.
      $vocabulary_name = substr($row->get('vocabulary_machine_name'), 0, 30);
      // Set property "id".
      $row->setSourceProperty('id', "menu_{$replaced_id}__vocab_{$vocabulary_name}");
      // Set property "label".
      $row->setSourceProperty('label', "menu_{$replaced_id}__vocab_{$vocabulary_name}");
      // Set property "mid".
      $row->setSourceProperty('mid', $mid);
    }
    return parent::prepareRow($row);
  }

  /**
   * {@inheritdoc}
   */
  protected function initializeIterator() {
    $results = $this->prepareQuery()->execute()->fetchAll();
    $processed_results = [];

    if ($results) {
      foreach ($results as $index => $result) {
        $value = unserialize($result['value']);
        if (!empty($value)) {
          $processed_results[] = $result;
        }
      }
    }

    return new \ArrayIterator($processed_results);
  }

  /**
   * {@inheritdoc}
   */
  public function count($refresh = FALSE) {
    // We alter the count function to have the iterator into consideration.
    return $this->initializeIterator()->count();
  }


}
